@@ -2,16 +2,16 @@
 # -*- coding:utf-8 -*-
 # @Author: Jialiang Shi
 try:
-    from urllib.parse import quote
+    from urllib.parse import quote_plus
 except ImportError:
-    from urllib import quote
+    from urllib import quote_plus
 
-from gerrit.changes.revision.drafts import Drafts
-from gerrit.changes.revision.comments import Comments
-from gerrit.changes.revision.files import Files
+from gerrit.changes.revision.drafts import GerritChangeRevisionDrafts
+from gerrit.changes.revision.comments import GerritChangeRevisionComments
+from gerrit.changes.revision.files import GerritChangeRevisionFiles
 
 
-class Revision(object):
+class GerritChangeRevision(object):
     def __init__(self, project, change, revision, gerrit):
         self.project = project
         self.change = change
@@ -27,7 +27,11 @@ class Revision(object):
         endpoint = "/changes/%s/revisions/%s/commit" % (self.change, self.revision)
         response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
         result = self.gerrit.decode_response(response)
-        return self.gerrit.projects.get(self.project).get_commit(result.get("commit"))
+        commit = result.get("commit")
+        if commit:
+            project = self.gerrit.projects.get(self.project)
+            if project:
+                return project.get_commit(commit)
 
     def get_description(self):
         """
@@ -51,7 +55,7 @@ class Revision(object):
                 "description": "Added Documentation"
             }
 
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             revision = change.get_revision('3848807f587dbd3a7e61723bbfbf1ad13ad5a00a')
             result = revision.set_description(input_)
 
@@ -154,7 +158,7 @@ class Revision(object):
                 }
             }
 
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             revision = change.get_revision('3848807f587dbd3a7e61723bbfbf1ad13ad5a00a')
             result = revision.set_review(input_)
 
@@ -181,7 +185,7 @@ class Revision(object):
                 "base" : "1234"
             }
 
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             revision = change.get_revision('3848807f587dbd3a7e61723bbfbf1ad13ad5a00a')
             result = revision.rebase(input_)
 
@@ -232,14 +236,14 @@ class Revision(object):
         """
         endpoint = "/changes/%s/revisions/%s/patch" % (self.change, self.revision)
 
-        if zip:
+        if zip_:
             endpoint += endpoint + "?zip"
 
         if download:
             endpoint += endpoint + "?download"
 
         if path:
-            endpoint += endpoint + "?path=%s" % quote(path, safe="")
+            endpoint += endpoint + "?path=%s" % quote_plus(path)
 
         response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
         result = self.gerrit.decode_response(response)
@@ -262,7 +266,8 @@ class Revision(object):
 
     def is_mergeable(self):
         """
-        Gets the method the server will use to submit (merge) the change and an indicator if the change is currently mergeable.
+        Gets the method the server will use to submit (merge) the change and an indicator
+        if the change is currently mergeable.
 
         :return:
         """
@@ -322,15 +327,51 @@ class Revision(object):
 
     @property
     def drafts(self):
-        return Drafts(change=self.change, revision=self.revision, gerrit=self.gerrit)
+        return GerritChangeRevisionDrafts(
+            change=self.change, revision=self.revision, gerrit=self.gerrit
+        )
 
     @property
     def comments(self):
-        return Comments(change=self.change, revision=self.revision, gerrit=self.gerrit)
+        return GerritChangeRevisionComments(
+            change=self.change, revision=self.revision, gerrit=self.gerrit
+        )
+
+    def list_robot_comments(self):
+        """
+        Lists the robot comments of a revision.
+
+        :return:
+        """
+        endpoint = "/changes/%s/revisions/%s/robotcomments" % (
+            self.change,
+            self.revision,
+        )
+        response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
+        result = self.gerrit.decode_response(response)
+        return result
+
+    def get_robot_comment(self, commit_id):
+        """
+        Retrieves a robot comment of a revision.
+
+        :param commit_id:
+        :return:
+        """
+        endpoint = "/changes/%s/revisions/%s/robotcomments/%s" % (
+            self.change,
+            self.revision,
+            commit_id,
+        )
+        response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
+        result = self.gerrit.decode_response(response)
+        return result
 
     @property
     def files(self):
-        return Files(change=self.change, revision=self.revision, gerrit=self.gerrit)
+        return GerritChangeRevisionFiles(
+            change=self.change, revision=self.revision, gerrit=self.gerrit
+        )
 
     def cherry_pick(self, input_):
         """
@@ -343,7 +384,7 @@ class Revision(object):
                 "destination" : "release-branch"
             }
 
-            change = gerrit.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
             revision = change.get_revision('3848807f587dbd3a7e61723bbfbf1ad13ad5a00a')
             result = revision.cherry_pick(input_)
 
@@ -369,3 +410,64 @@ class Revision(object):
         response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
         result = self.gerrit.decode_response(response)
         return result
+
+    def list_votes(self, account):
+        """
+        Lists the votes for a specific reviewer of the revision.
+
+        :param account: account id or username
+        :return:
+        """
+        endpoint = "/changes/%s/revisions/%s/reviewers/%s/votes/" % (
+            self.change,
+            self.revision,
+            account,
+        )
+        response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
+        result = self.gerrit.decode_response(response)
+        return result
+
+    def delete_vote(self, account, label, input_=None):
+        """
+        Deletes a single vote from a revision. The deletion will be possible only
+        if the revision is the current revision. By using this endpoint you can prevent
+        deleting the vote (with same label) from a newer patch set by mistake.
+        Note, that even when the last vote of a reviewer is removed the reviewer itself is still listed on the change.
+
+        .. code-block:: python
+
+            input_ = {
+                "notify": "NONE"
+            }
+
+            change = client.changes.get('myProject~stable~I10394472cbd17dd12454f229e4f6de00b143a444')
+            revision = change.get_revision("0f4f97b5af9a965e082fb8cde082c5f1ba2fe930")
+            revision.delete_vote('Code-Review', input_)
+            # or
+            revision.delete_vote('Code-Review')
+
+        :param account:
+        :param label:
+        :param input_: the DeleteVoteInput entity,
+          https://gerrit-review.googlesource.com/Documentation/rest-api-changes.html#delete-vote-input
+        :return:
+        """
+        if input_ is None:
+            endpoint = "/changes/%s/revisions/%s/reviewers/%s/votes/%s" % (
+                self.change,
+                self.revision,
+                account,
+                label,
+            )
+            self.gerrit.requester.delete(self.gerrit.get_endpoint_url(endpoint))
+        else:
+            endpoint = "/changes/%s/revisions/%s/reviewers/%s/votes/%s/delete" % (
+                self.change,
+                self.revision,
+                account,
+                label,
+            )
+            base_url = self.gerrit.get_endpoint_url(endpoint)
+            self.gerrit.requester.post(
+                base_url, json=input_, headers=self.gerrit.default_headers
+            )
