@@ -2,12 +2,12 @@
 # -*- coding:utf-8 -*-
 # @Author: Jialiang Shi
 from packaging.version import parse
-from gerrit.projects.branches import Branches
-from gerrit.projects.tags import Tags
-from gerrit.projects.commit import Commit
-from gerrit.projects.dashboards import Dashboards
-from gerrit.projects.labels import Labels
-from gerrit.projects.webhooks import Webhooks
+from gerrit.projects.branches import GerrirProjectBranches
+from gerrit.projects.tags import GerrirProjectTags
+from gerrit.projects.commit import GerrirProjectCommit
+from gerrit.projects.dashboards import GerrirProjectDashboards
+from gerrit.projects.labels import GerrirProjectLabels
+from gerrit.projects.webhooks import GerrirProjectWebHooks
 from gerrit.changes.change import GerritChange
 from gerrit.utils.models import BaseModel
 from gerrit.utils.exceptions import UnsupportMethod
@@ -16,10 +16,8 @@ from gerrit.utils.exceptions import UnsupportMethod
 class GerritProject(BaseModel):
     def __init__(self, **kwargs):
         super(GerritProject, self).__init__(**kwargs)
-        self.attributes = ["id", "name", "state", "web_links", "gerrit"]
 
-    @property
-    def description(self):
+    def get_description(self):
         """
         Retrieves the description of a project.
 
@@ -37,10 +35,10 @@ class GerritProject(BaseModel):
         .. code-block:: python
 
             input_ = {
-                "description": "Plugin for Gerrit that handles the replication.",,
+                "description": "Plugin for Gerrit that handles the replication.",
                 "commit_message": "Update the project description"
             }
-            project = gerrit.projects.get('myproject')
+            project = client.projects.get('myproject')
             result = project.set_description(input_)
 
         :param input_: the ProjectDescriptionInput entity,
@@ -73,8 +71,7 @@ class GerritProject(BaseModel):
         endpoint = "/projects/%s/delete-project~delete" % self.id
         self.gerrit.requester.post(self.gerrit.get_endpoint_url(endpoint))
 
-    @property
-    def parent(self):
+    def get_parent(self):
         """
         Retrieves the name of a project’s parent project. For the All-Projects root project an empty string is returned.
 
@@ -95,7 +92,7 @@ class GerritProject(BaseModel):
                 "parent": "Public-Plugins",
                 "commit_message": "Update the project parent"
             }
-            project = gerrit.projects.get('myproject')
+            project = client.projects.get('myproject')
             result = project.set_parent(input_)
 
         :param input_: The ProjectParentInput entity,
@@ -110,8 +107,7 @@ class GerritProject(BaseModel):
         result = self.gerrit.decode_response(response)
         return result
 
-    @property
-    def HEAD(self):
+    def get_HEAD(self):
         """
         Retrieves for a project the name of the branch to which HEAD points.
 
@@ -131,7 +127,7 @@ class GerritProject(BaseModel):
             input_ = {
                 "ref": "refs/heads/stable"
             }
-            project = gerrit.projects.get('myproject')
+            project = client.projects.get('myproject')
             result = project.set_HEAD(input_)
 
         :param input_: The HeadInput entity,
@@ -146,8 +142,7 @@ class GerritProject(BaseModel):
         result = self.gerrit.decode_response(response)
         return result
 
-    @property
-    def config(self):
+    def get_config(self):
         """
         Gets some configuration information about a project.
         Note that this config info is not simply the contents of project.config; it generally contains fields that may
@@ -180,7 +175,7 @@ class GerritProject(BaseModel):
                 "submit_type": "REBASE_IF_NECESSARY",
                 "state": "ACTIVE"
             }
-            project = gerrit.projects.get('myproject')
+            project = client.projects.get('myproject')
             result = project.set_config(input_)
 
         :param input_: the ConfigInput entity,
@@ -215,7 +210,7 @@ class GerritProject(BaseModel):
             input_ = {
                 "show_progress": "true"
             }
-            project = gerrit.projects.get('myproject')
+            project = client.projects.get('myproject')
             result = project.run_garbage_collection(input_)
 
         :param input_: the GCInput entity,
@@ -243,7 +238,7 @@ class GerritProject(BaseModel):
                 ],
                 "reason": "Violates IP"
             }
-            project = gerrit.projects.get('myproject')
+            project = client.projects.get('myproject')
             result = project.ban_commits(input_)
 
         :param input_: the BanInput entity,
@@ -258,8 +253,7 @@ class GerritProject(BaseModel):
         result = self.gerrit.decode_response(response)
         return result
 
-    @property
-    def access_rights(self):
+    def get_access_rights(self):
         """
         Lists the access rights for a single project.
 
@@ -289,7 +283,8 @@ class GerritProject(BaseModel):
 
     def create_change(self, input_):
         """
-        Create Change for review.
+        Create Change for review. This endpoint is functionally equivalent to create change in the change API,
+        but it has the project name in the URL, which is easier to route in sharded deployments.
         support this method since v3.3.0
 
         .. code-block:: python
@@ -301,7 +296,7 @@ class GerritProject(BaseModel):
                 "status": "NEW"
             }
 
-            project = gerrit.projects.get('myproject')
+            project = client.projects.get('myproject')
             result = project.create_change(input_)
 
         :param input_:
@@ -368,7 +363,7 @@ class GerritProject(BaseModel):
                 "index_children": "true"
                 "async": "true"
             }
-            project = gerrit.projects.get('myproject')
+            project = client.projects.get('myproject')
             result = project.index(input_)
 
         :param input_: the IndexProjectInput entity,
@@ -406,7 +401,7 @@ class GerritProject(BaseModel):
                 }
             }
 
-            project = gerrit.projects.get('myproject')
+            project = client.projects.get('myproject')
             result = project.check_consistency(input_)
 
         :param input_: the CheckProjectInput entity,
@@ -428,7 +423,7 @@ class GerritProject(BaseModel):
 
         :return:
         """
-        return Branches(self.id, self.gerrit)
+        return GerrirProjectBranches(self.id, self.gerrit)
 
     @property
     def child_projects(self):
@@ -440,7 +435,7 @@ class GerritProject(BaseModel):
         endpoint = "/projects/%s/children/" % self.id
         response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
         result = self.gerrit.decode_response(response)
-        return [self.gerrit.projects.get(item.get("id")) for item in result]
+        return result
 
     @property
     def tags(self):
@@ -449,7 +444,7 @@ class GerritProject(BaseModel):
 
         :return:
         """
-        return Tags(self.id, self.gerrit)
+        return GerrirProjectTags(self.id, self.gerrit)
 
     def get_commit(self, commit):
         """
@@ -460,7 +455,7 @@ class GerritProject(BaseModel):
         endpoint = "/projects/%s/commits/%s" % (self.id, commit)
         response = self.gerrit.requester.get(self.gerrit.get_endpoint_url(endpoint))
         result = self.gerrit.decode_response(response)
-        return Commit.parse(result, project=self.id, gerrit=self.gerrit)
+        return GerrirProjectCommit.parse(result, project=self.id, gerrit=self.gerrit)
 
     @property
     def dashboards(self):
@@ -469,10 +464,9 @@ class GerritProject(BaseModel):
 
         :return:
         """
-        return Dashboards(project=self.id, gerrit=self.gerrit)
+        return GerrirProjectDashboards(project=self.id, gerrit=self.gerrit)
 
-    @property
-    def labels(self):
+    def get_labels(self):
         """
         gerrit labels or gerrit labels operations
 
@@ -485,7 +479,7 @@ class GerritProject(BaseModel):
             result = self.gerrit.decode_response(response)
             return result.get("labels")
         else:
-            return Labels(project=self.id, gerrit=self.gerrit)
+            return GerrirProjectLabels(project=self.id, gerrit=self.gerrit)
 
     @property
     def webhooks(self):
@@ -494,4 +488,4 @@ class GerritProject(BaseModel):
 
         :return:
         """
-        return Webhooks(project=self.id, gerrit=self.gerrit)
+        return GerrirProjectWebHooks(project=self.id, gerrit=self.gerrit)
